/***************************************************************************
*   Copyright (C) 2005 by David Voswinkel                                 *
*   d.voswinkel@netcologne.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "kbtscoworker.h"
#include "trayicon.h"
#include "kbtsco.h"

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include <netinet/ip.h>
#include <netinet/udp.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <bluetooth/sco.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>
#include <alsa/asoundlib.h>

#include <qstring.h>
#include <qtimer.h>
#include <qfile.h>
#include <qptrqueue.h>
#include <qapplication.h>

#include <kdebug.h>
#include <klocale.h>


#define SNDRV_BT_SCO_IOCTL_SET_SCO_SOCKET _IOW ('H', 0x10, int)

#ifndef SND_HWDEP_IFACE_EMUX_WAVETABLE
#define SND_HWDEP_IFACE_EMUX_WAVETABLE (SND_HWDEP_IFACE_USX2Y + 1)
#endif

#ifndef SND_HWDEP_IFACE_BLUETOOTH
#define SND_HWDEP_IFACE_BLUETOOTH (SND_HWDEP_IFACE_EMUX_WAVETABLE + 1)
#endif

#ifndef SNDRV_HWDEP_IFACE_BT_SCO
#define SNDRV_HWDEP_IFACE_BT_SCO (SND_HWDEP_IFACE_BLUETOOTH + 1)
#endif


KBtscoWorker::KBtscoWorker (TrayIcon *p) : m_parent (p)
{
    is_connected = FALSE;
    first = NULL;
    bt_dev = 0;
    current_headset = NULL;
    channel = 0;
    force_cancel = FALSE;
}


KBtscoWorker::~KBtscoWorker ()
{
    kdDebug (200011) << "KBtscoWorker::~KBtscoWorker" << endl;
    if (running ()) {
        terminate ();
        wait ();
        kdDebug (200011) << "KBtscoWorker::~KBtscoWorker thread terminated" << endl;
    }

    if (connected ()){
        kdDebug (200011) << "KBtscoWorker::~KBtscoWorker call disconnect" << endl;
        btscoClose ();
    }
    kdDebug (200011) << "KBtscoWorker::~KBtscoWorker done" << endl;
}

void KBtscoWorker::addCommand (KBtscoWorkerCommand * cmd)
{
    command_mutex.lock ();
    command_queue.enqueue (cmd);
    command_mutex.unlock ();
    command_cond.wakeOne ();
}

KBtscoWorkerCommand * KBtscoWorker::command ()
{
    KBtscoWorkerCommand *cmd = 0;
    command_mutex.lock ();
    cmd = command_queue.dequeue ();
    command_mutex.unlock ();
    return cmd;
}


void KBtscoWorker::run ()
{
    KBtscoWorkerCommand *cmd = 0;
    while (TRUE) {
        if (!cmd)
            cmd = command ();
        if (cmd) {
            kdDebug (200011) << "KBtscoWorker::run cmd=" << cmd->command () << endl;

            switch (cmd->command ()) {
                case KBtscoWorkerCommand::Connect:
                    if (btscoOpen())
                        btscoWork();
                    else{
                        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Disconnected);
                        QApplication::postEvent (m_parent, e);
                    }
                    if (cmd)
                        delete cmd;
                    cmd = 0;
                    break;
                case KBtscoWorkerCommand::Disconnect:
                    btscoClose ();
                    delete cmd;
                    cmd = 0;
                    break;
                default:
                    kdDebug (200011) << "KBtscoWorker::run unhandled cmd=" << cmd->command () << endl;
                    break;
            }

        } else {
            kdDebug (200011) << "KBtscoWorker::run no cmd going to sleep" << endl;
            command_cond.wait ();
        }
    }
}



void
KBtscoWorker::setBaddr (QString & s)
{
    baddr = s;
    kdDebug (200011) << "KBtscoWorker::setAddr " << baddr << endl;
}

void
KBtscoWorker::setChannel (int c)
{
    channel = c&0xff;
    kdDebug (200011) << "KBtscoWorker::setChannel " << channel << endl;
}

void
KBtscoWorker::forceCancel ()
{
    force_cancel = TRUE;
    kdDebug (200011) << "KBtscoWorker::forceCancel " << endl;
}

bool KBtscoWorker::btscoOpen ()
{
    int err;
    force_cancel = FALSE;
    kdDebug (200011) << "KBtscoWorker::btscoOpen" << endl;
    if (connected()){
        kdDebug (200011) << "KBtscoWorker::btscoOpen error: connected" << endl;
        return TRUE;
    }
    KBtscoStatusEvent *e;
    if (findDevice(&card, &dev)) {
        perror("Can't find device. Bail");
        return FALSE;
    }
    kdDebug (200011)    << "KBtscoWorker::btscoOpen Device is "
            << card << ":" <<  dev  << endl;

    sprintf(hwdep_name, "hw:%i,%i", card, dev);

    if (checkBluetoothVoice(bt_dev) < 0)
        return FALSE;

    current_headset = headsetNew();
    hci_devba(bt_dev, &current_headset->local);
    str2ba(baddr.latin1(), &current_headset->bdaddr);

    if (channel != 0x00){
        current_headset->channel = channel;
        kdDebug (200011) << "KBtscoWorker::btscoOpen btsco fixed channel="
                <<  channel << endl;
    } else {
       current_headset->channel = detectChannel(&current_headset->bdaddr);
       kdDebug (200011) << "KBtscoWorker::btscoOpen btsco lookup channel="
               <<   current_headset->channel  << endl;
    }
    if (force_cancel){
        kdDebug (200011) << "KBtscoWorker::btscoOpen cancel requested" << endl;
        return FALSE;
    }
    /* open hwdep on audio device */
    if ((err = snd_hwdep_open(&current_headset->handle, hwdep_name, O_RDWR)) < 0) {
        QString msg = i18n("Unable to connect to Audio Device");
        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Error,msg);
        QApplication::postEvent (m_parent, e);
        kdDebug (200011) << "KBtscoWorker::btscoOpen btsco open ("
                << card << "-" << dev << "): "
                << snd_strerror(err) << endl;
                return FALSE;
    }
    e = new KBtscoStatusEvent (KBtscoStatusEvent::Connected);
    QApplication::postEvent (m_parent, e);
    is_connected = TRUE;
    kdDebug (200011) << "KBtscoWorker::btscoOpen done" << endl;
    return TRUE;
}


void KBtscoWorker::btscoClose ()
{
    kdDebug (200011) << "KBtscoWorker::btscoClose" << endl;
    if (!connected()){
        kdDebug (200011) << "KBtscoWorker::btscoClose error: not connected" << endl;
        return;
    }
    struct s_headset *current;
    current = first;
    while (current != NULL) {
        struct s_headset *next = current->next;
        kdDebug (200011) << "KBtscoWorker::btscoClose destroy headset=" << current << endl;
        headsetDestroy(current);
        current = next;
    }

    KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Disconnected);
    QApplication::postEvent (m_parent, e);
    is_connected = FALSE;
    kdDebug (200011) << "KBtscoWorker::btscoClose done" << endl;
}


bool KBtscoWorker::btscoWork ()
{
    KBtscoWorkerCommand * cmd = 0;
    bool do_stop = FALSE;
    struct pollfd pfds[16];
    short revents;
    int nfds;
    int local_err = 0;

    do {
        nfds = 0;
        cmd = 0;
        cmd = command ();
        if (cmd) {
            switch (cmd->command ()) {
                case KBtscoWorkerCommand::Volume:
                    kdDebug (200011) << "KBtscoWorker::btscoWork cmd=KBtscoWorkerCommand::Volume" << endl;
                    volume = cmd->value ();
                    // todo
                    delete cmd;
                    cmd = 0;
                    break;
                case KBtscoWorkerCommand::Disconnect:
                    kdDebug (200011) << "KBtscoWorker::btscoWork cmd=KBtscoWorkerCommand::Disconnect" << endl;
                    do_stop = TRUE;
                    /* forward to global handling */
                    addCommand (cmd);
                    break;
                default:
                    break;
            }
        }
        if (do_stop)
            break;

        /* set up data polling description */
        for (current_headset = first; current_headset != NULL; current_headset = current_headset->next) {
            if (current_headset->rfcomm_fd == -1) {
                /* connect rfcomm control channel */
                if ((current_headset->rfcomm_fd = rfcommConnect(
                     &current_headset->local,
                &current_headset->bdaddr,
                current_headset->channel)) < 0) {
                    kdDebug (200011) << "KBtscoWorker::btscoWork Can't connect RFCOMM channel" << endl;
                    local_err++;
                    if (force_cancel)
                        break;
                    if (5 == local_err){
                        QString msg = i18n("Can't connect RFCOMM channel");
                        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Error,msg);
                        QApplication::postEvent (m_parent, e);
                        do_stop = TRUE;
                    }

                } else {
                    kdDebug (200011) << "KBtscoWorker::btscoWork RFCOMM channel "
                        << current_headset->channel << " connected"  << endl;
                }
            }
            if (current_headset->rfcomm_fd != -1) {
                pfds[nfds].fd = current_headset->rfcomm_fd;
                pfds[nfds++].events = POLLIN;
            }
            if (current_headset->handle != NULL) {
                /* polling data from hwdep interface */
                nfds += snd_hwdep_poll_descriptors(current_headset->handle, &pfds[nfds], 1);
            }
        }
        if (do_stop)
            break;

        if (nfds == 0) {
            kdDebug (200011) << "KBtscoWorker::btscoWork nfds == 0, going to sleep " << endl;
            this->sleep(3);
            continue;
        }
        if (poll(pfds, nfds, 1000) <= 0)
            continue;


        for (current_headset = first; current_headset != NULL; current_headset = current_headset->next) {
            int j;
            for (j = 0; j < nfds; j++) {
                if (pfds[j].fd == current_headset->rfcomm_fd) {
                    if (do_stop)
                        break;
                    if (pfds[j].revents & POLLIN)
                        headsetFromBluetooth(current_headset);
                    continue;
                }
                /* Volume polling (sound card) */
                if (!snd_hwdep_poll_descriptors_revents (current_headset->handle, &pfds[j], 1,(unsigned short int *) &revents) && revents & POLLIN)
                    headsetVolumeFromCard (current_headset);
            }
        }
    } while (!do_stop || !force_cancel);
    kdDebug (200011) << "KBtscoWorker::btscoWork leaving work loop" << endl;
    return TRUE;
}



int KBtscoWorker::rfcommConnect(bdaddr_t *src, bdaddr_t *dst, uint8_t channel)
{
    struct sockaddr_rc addr;
    int sk;

    sk = socket(PF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
    if (sk < 0)
        return -1;

    memset(&addr, 0, sizeof(addr));
    addr.rc_family = AF_BLUETOOTH;
    bacpy(&addr.rc_bdaddr, src);

    if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
        close(sk);
        return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.rc_family = AF_BLUETOOTH;
    bacpy(&addr.rc_bdaddr, dst);
    addr.rc_channel = channel;

    if (connect(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
        close(sk);
        return -1;
    }

    return sk;
}

int KBtscoWorker::scoConnect(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t *mtu)
{
    struct sockaddr_sco addr;
    struct sco_conninfo conn;
    struct sco_options opts;
    socklen_t size;
    int sk;

    sk = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_SCO);
    if (sk < 0)
        return -1;

    memset(&addr, 0, sizeof(addr));
    addr.sco_family = AF_BLUETOOTH;
    bacpy(&addr.sco_bdaddr, src);

    if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
        close(sk);
        return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sco_family = AF_BLUETOOTH;
    bacpy(&addr.sco_bdaddr, dst);

    if (connect(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
        close(sk);
        return -1;
    }

    size = sizeof(conn);

    if (getsockopt(sk, SOL_SCO, SCO_CONNINFO, &conn, &size) < 0) {
        close(sk);
        return -1;
    }

    size = sizeof(opts);

    if (getsockopt(sk, SOL_SCO, SCO_OPTIONS, &opts, &size) < 0) {
        close(sk);
        return -1;
    }

    if (handle)
        *handle = conn.hci_handle;

    if (mtu)
        *mtu = opts.mtu;

    return sk;
}

int KBtscoWorker::BluetoothScoSetFileDescriptor(snd_hwdep_t * handle, int sco_fd)
{
    if (snd_hwdep_ioctl(handle, SNDRV_BT_SCO_IOCTL_SET_SCO_SOCKET, (void *) sco_fd) < 0) {
        KBtscoStatusEvent *e;
        QString msg = i18n("Unable to set SCO filedescriptior, will disconnect device.");
        e = new KBtscoStatusEvent (KBtscoStatusEvent::Error,msg);
        QApplication::postEvent (m_parent, e);
        e = new KBtscoStatusEvent (KBtscoStatusEvent::DisconnectedSCO);
        QApplication::postEvent (m_parent, e);
        return -1;
    }
    return 0;
}

int KBtscoWorker::findDevice(int *cardP, int *devP)
{
    snd_ctl_t *ctl_handle;
    snd_ctl_card_info_t *card_info;
    snd_hwdep_info_t *hwdep_info;

    int card;
    int dev;
    int err;
    char card_id[32];

    ctl_handle = NULL;

    snd_ctl_card_info_alloca(&card_info);
    snd_hwdep_info_alloca(&hwdep_info);

    for (card = 0; card < 7; card++) {
        *cardP = card;

        if (ctl_handle) {
            snd_ctl_close(ctl_handle);
            ctl_handle = NULL;
        }
                // Get control handle for selected card
        sprintf(card_id, "hw:%i", card);
        if ((err = snd_ctl_open(&ctl_handle, card_id, 0)) < 0) {
            kdDebug (200011) << "KBtscoWorker::findDevice control open ("
                    << card_id << "): "
                    <<  snd_strerror(err) << endl;
            return -1;
        }
                // Read control hardware info from card
        if ((err = snd_ctl_card_info(ctl_handle, card_info)) < 0) {
            kdDebug (200011) << "KBtscoWorker::findDevice control hardware info ("
                    << card_id << "): "
                    <<  snd_strerror(err) << endl;
            continue;
        }
                //if (strcmp(snd_ctl_card_info_get_driver(card_info),"BT SCO (d)"))
                //    continue;
        dev = -1;
        err = 1;
        while (1) {
            int if_type;
            if (snd_ctl_hwdep_next_device(ctl_handle, &dev) < 0)
                kdDebug (200011) << "KBtscoWorker::findDevice hwdep next device ("
                        << card_id << "): "
                        <<  snd_strerror(err) << endl;
            if (dev < 0)
                break;
            snd_hwdep_info_set_device(hwdep_info, dev);
            if (snd_ctl_hwdep_info(ctl_handle, hwdep_info) < 0) {
                if (err != -ENOENT)
                    kdDebug (200011) << "KBtscoWorker::findDevice control hwdep info ("
                            << card_id << "): "
                            <<  snd_strerror(err) << endl;
                continue;
            }
            if_type = snd_hwdep_info_get_iface(hwdep_info);
            if (if_type == SNDRV_HWDEP_IFACE_BT_SCO || if_type==12) {
                snd_ctl_close(ctl_handle);
                *devP = dev;
                return 0;
            }
        }
    }
    if (ctl_handle)
        snd_ctl_close(ctl_handle);

    return -1;
}

int KBtscoWorker::detectChannel(bdaddr_t * bdaddr)
{
    uuid_t group;
    bdaddr_t interface;
    sdp_list_t *attrid, *search, *seq, *next;
    uint32_t range = 0x0000ffff;
    sdp_session_t *sess;
    int channel = 1;
    int searchresult;
    kdDebug (200011) << "KBtscoWorker::detectChannel" << endl;
    bacpy(&interface, BDADDR_ANY);

    sdp_uuid16_create(&group, 0x1108);
    sess = sdp_connect(&interface, bdaddr, SDP_RETRY_IF_BUSY);
    if (!sess) {
        kdDebug (200011) << "KBtscoWorker::detectChannel Failed to connect to SDP server: "
                << strerror(errno) << " Assuming channel " << channel << endl;

        QString msg = i18n("Failed to connect to SDP server. Assuming channel 1.");
        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Error,msg);
        QApplication::postEvent (m_parent, e);
        return channel;
    }

    attrid = sdp_list_append(0, &range);
    search = sdp_list_append(0, &group);
    searchresult = sdp_service_search_attr_req(sess, search,
                                               SDP_ATTR_REQ_RANGE, attrid, &seq);
    sdp_list_free(attrid, 0);
    sdp_list_free(search, 0);

    if (searchresult) {
        kdDebug (200011) << "KBtscoWorker::detectChannel Service Search failed: "
                << strerror(errno) << " Assuming channel " << channel << endl;

        QString msg = i18n("Service Search failed. Assuming channel 1." );
        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Error,msg);
        QApplication::postEvent (m_parent, e);

        sdp_close(sess);
        return channel;
    }

    for (; seq; seq = next) {
        sdp_record_t *rec = (sdp_record_t *) seq->data;
        sdp_list_t *list = 0;
        if (sdp_get_access_protos(rec, &list) == 0) {
            channel = sdp_get_proto_port(list, RFCOMM_UUID);
        }
        next = seq->next;
        free(seq);
        sdp_record_free(rec);
    }

    sdp_close(sess);
    return channel;
}

int KBtscoWorker::headsetButton(struct s_headset *headset)
{
    uint16_t sco_handle, sco_mtu;

    if (headset == NULL)
        return 0;
    if (headset->sco_fd != -1) {
        /* close bt_sco audio handle */
        BluetoothScoSetFileDescriptor(headset->handle, -1);
        /* disconnect SCO stream */
        close(headset->sco_fd);
        headset->sco_fd = -1;
        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::DisconnectedSCO);
        QApplication::postEvent (m_parent, e);
        return 1;
    }
    kdDebug (200011) << "KBtscoWorker::headsetButton opened hwdep" << endl;
    /* connect sco stream */
    if ((headset->sco_fd = scoConnect(&headset->local, &headset->bdaddr, &sco_handle, &sco_mtu)) < 0) {
        QString msg = i18n("Can't connect SCO audio channel. Will disconnect device.");
        KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::Error,msg);
        QApplication::postEvent (m_parent, e);
        KBtscoWorkerCommand *c = new KBtscoWorkerCommand (KBtscoWorkerCommand::Disconnect, 0);
        addCommand (c);
        return 1;
    }
    KBtscoStatusEvent *e = new KBtscoStatusEvent (KBtscoStatusEvent::ConnectedSCO);
    QApplication::postEvent (m_parent, e);
    kdDebug (200011) << "KBtscoWorker::headsetButton connected SCO channel" << endl;
    //write(headset->rfcomm_fd, "RING\r\n", 6);
    BluetoothScoSetFileDescriptor(headset->handle, headset->sco_fd);
    return 1;
}

struct s_headset * KBtscoWorker::headsetNew(void)
{
    struct s_headset *headset;
    headset = ( struct s_headset *)malloc (sizeof(struct s_headset));
    if (headset == NULL)
        return NULL;
    headset->sco_fd = -1;
    headset->rfcomm_fd = -1;
    headset->handle = NULL;
    headset->last_volumes[0] = 0;
    headset->last_volumes[1] = 0;
    headset->next = first;
    first = headset;
    return headset;
}

int KBtscoWorker::headsetVolumeFromCard(struct s_headset *headset)
{
    int len;
    char line[100];

    len = snd_hwdep_read(headset->handle, headset->volumes, sizeof(headset->volumes));
    if (len != sizeof(headset->volumes))
        return 0;

    kdDebug (200011) << "KBtscoWorker::headsetVolumeFromCard"
            << " volume speaker: " <<headset->volumes[0]
            << " mic: " <<  headset->volumes[1] << endl;
    if (headset->volumes[0] != headset->last_volumes[0]) {
        sprintf(line, "+VGS=%d\r", headset->volumes[0]);
        write(headset->rfcomm_fd, line, strlen(line));
        headset->last_volumes[0] = headset->last_volumes[0];
    }
    if (headset->volumes[1] != headset->last_volumes[1]) {
        sprintf(line, "+VGM=%d\r", headset->volumes[1]);
        write(headset->rfcomm_fd, line, strlen(line));
        headset->last_volumes[1] = headset->last_volumes[1];
    }
    return 1;
}

int KBtscoWorker::headsetSpeaker(struct s_headset *headset)
{
    kdDebug (200011) << "KBtscoWorker::headsetSpeaker "
            << "Sending up speaker change " << headset->volumes[0]
            << endl;
    snd_hwdep_write(headset->handle, headset->volumes, sizeof (headset->volumes));
    return 1;
}

int KBtscoWorker::headsetMicro(struct s_headset *headset)
{
    kdDebug (200011) << "KBtscoWorker::headsetMicro "
            << "Sending up microphone change " << headset->volumes[1]
            << endl;
    snd_hwdep_write(headset->handle, headset->volumes, sizeof (headset->volumes));
    return 1;
}

int KBtscoWorker::headsetFromBluetooth(struct s_headset *headset)
{
    unsigned char buf[2048];
    int rlen;
    int opdone;

    opdone = 0;
    rlen = read(headset->rfcomm_fd, buf, sizeof(buf) - 1);
    if (rlen <= 0)
        return 0;
    buf [rlen] = 0;
    kdDebug (200011) << "KBtscoWorker::headsetFromBluetooth "
            << "recieved " <<  buf << endl;
    if (strstr((const char*)buf, "AT+BVRA="   ))
        opdone = headsetButton(headset);
    else if (strstr((const char*)buf, "AT+CKPD=200"))
        opdone = headsetButton(headset);
    else if (strstr((const char*)buf, "AT+CHUP"    ))
        opdone = headsetButton(headset);
    else if (strstr((const char*)buf, "AT+CIND=?"  ))
        opdone = headsetButton(headset);
    else if (sscanf ((const char*)buf, "AT+VGS=%d", &headset->volumes[0]) == 1)
        opdone = headsetSpeaker (headset);
    else if (sscanf ((const char*)buf, "AT+VGM=%d", &headset->volumes[1]) == 1)
        opdone = headsetMicro   (headset);
    if (opdone == 1)
        /* tell them we recieved */
        write(headset->rfcomm_fd, "\r\nOK\r\n", 6);
    else
        write(headset->rfcomm_fd, "\r\nERROR\r\n", 9);
    return 1;
}

void KBtscoWorker::headsetDestroy(struct s_headset *headset)
{
    if (headset == NULL)
        return;
    if (headset->sco_fd != -1) {
        BluetoothScoSetFileDescriptor(headset->handle, -1);
        close(headset->sco_fd);
    }
    this->sleep(1);
    if (headset->rfcomm_fd != -1)
        close(headset->rfcomm_fd);
    if (headset->handle != NULL)
        snd_hwdep_close(headset->handle);
    headset->sco_fd = -1;
    headset->rfcomm_fd = -1;
    headset->handle = NULL;
}


int KBtscoWorker::checkBluetoothVoice(int dev)
{
    int dd;
    uint16_t vs;
    /* check voice settings. in this version we only support mu-law */
    dd = hci_open_dev(dev);
    hci_read_voice_setting(dd, &vs, 1000);
    vs = htobs(vs);
    kdDebug (200011) << "KBtscoWorker::checkBluetoothVoice "
            << "Voice setting: " <<  vs << endl;
    close(dd);
    if (vs != 0x060) {
        kdDebug (200011) << "KBtscoWorker::checkBluetoothVoice "
                <<  "The voice setting must be 0x060" << endl;
        return -1;
    }
    return 0;
}
