
/***************************************************************************
 *   Copyright (C) 2005 by David Voswinkel                                 *
 *   d.voswinkel@netcologne.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KBTSCOWORKER_H_
#define _KBTSCOWORKER_H_

#include <qthread.h>
#include <qevent.h>
#include <qmutex.h>
#include <qptrqueue.h>
#include <qwaitcondition.h>


#include <netinet/ip.h>
#include <netinet/udp.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <bluetooth/sco.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>
#include <alsa/asoundlib.h>

class TrayIcon;
class KBtsco;


struct s_headset {
    bdaddr_t local;
    bdaddr_t bdaddr;
    uint8_t channel;
    int rfcomm_fd;
    int sco_fd;
    snd_hwdep_t *handle;
    int volumes[2];
    int last_volumes[2];
    struct s_headset *next;
};

class KBtscoStatusEvent: public QCustomEvent
{
public:
    KBtscoStatusEvent (int type, QString string = NULL, int val = 0)
            : QCustomEvent (type), s (string), v (val)
    {}

    QString string () const
    {
        return s;
    }
    int value () const
    {
        return v;
    }
    typedef enum Type
    {
      Message= 65000,
      Error,
      Volume,
      Connecting,
      Connected,
      ConnectedSCO,
      DisconnectedSCO,
      Disconnected,
    }
    Type;
private:
    QString s;
    int v;
};


class KBtscoWorkerCommand
{
public:
    KBtscoWorkerCommand (int c, int v): cmd (c), val (v)
    {}
    int value () const
    {
        return val;
    }
    int command () const
    {
        return cmd;
    }

    typedef enum Cmd
    { Volume,
      Connect,
      Disconnect,
    }
    Cmd;

private:
    int cmd;
    int val;
};




class KBtscoWorker: public QThread
{

public:
    KBtscoWorker (TrayIcon * p);
    ~KBtscoWorker ();
    bool connected ()
    {
        return is_connected;
    }
    void addCommand (KBtscoWorkerCommand * cmd);
    void setBaddr (QString & s);
    QString & addr ()
    {
        return baddr;
    }
    int currentVolume () const
    {
        return volume;
    }
    void setChannel (int c);
    void forceCancel ();

    virtual void run ();

private:

    KBtscoWorkerCommand *command ();
    bool btscoOpen ();
    void btscoClose ();
    bool btscoWork ();
    int rfcommConnect(bdaddr_t *src, bdaddr_t *dst, uint8_t channel);
    int scoConnect(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t *mtu);
    int BluetoothScoSetFileDescriptor(snd_hwdep_t * handle, int sco_fd);
    int findDevice(int *cardP, int *devP);
    int detectChannel(bdaddr_t * bdaddr);
    int headsetButton(struct s_headset *headset);
    struct s_headset * headsetNew(void);
    int headsetVolumeFromCard(struct s_headset *headset);
    int headsetSpeaker(struct s_headset *headset);
    int headsetMicro(struct s_headset *headset);
    int headsetFromBluetooth(struct s_headset *headset);
    void headsetDestroy(struct s_headset *headset);
    int checkBluetoothVoice(int dev);
    int dev;
    uint8_t channel;
    int card;
    int bt_dev;
    char hwdep_name[16];
    bool is_connected;
    bool force_cancel;
    int blue_sock;
    int volume;
    QString baddr;
    TrayIcon *m_parent;
    QPtrQueue < KBtscoWorkerCommand > command_queue;
    QMutex command_mutex;
    QWaitCondition command_cond;
    struct s_headset *first;
    struct s_headset *current_headset;

};
#endif
